/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui.dnd;

import java.util.*;

import org.eclipse.jface.viewers.*;
import org.eclipse.swt.dnd.*;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.internal.Utils;
import org.eclipse.ui.part.PluginTransfer;
import org.eclipse.ui.part.PluginTransferData;

/**
 * Drag source listener for <code>ITargetResource</code>
 */
public class TargetResourceDragListener extends DragSourceAdapter {

	private StructuredViewer viewer;

	public static int getSupportedOperations() {
		return DND.DROP_MOVE | DND.DROP_COPY;
	}
	
	public TargetResourceDragListener(StructuredViewer viewer) {
		this.viewer = viewer;
	}

	/**
	 * Method declared on DragSourceListener
	 */
	public void dragFinished(DragSourceEvent event) {
		if (!event.doit)
			return;
		//if the gadget was moved, remove it from the source viewer
		if (event.detail == DND.DROP_MOVE) {
			// Even though a move was performed, don't delete the source of the drag.
			// This is done because the drop may be run in the background
			// so the source needs to stick around.
			// Also, the source is remote and the delete would be long running
			// (i.e. requiring a progress monitor)
		}
	}

	/**
	 * Method declared on DragSourceListener
	 */
	public void dragSetData(DragSourceEvent event) {
		ITargetResource[] resources = getResources(viewer.getSelection());
		
        if (PluginTransfer.getInstance().isSupportedType(event.dataType)) {
            byte[] segmentData = TargetResourceTransfer.getInstance().toByteArray(resources);
            // TODO: HACK: The Id is from another plugin
            event.data = new PluginTransferData("org.eclipse.target.ide.targetResourceDrop", segmentData);
            return;
        }
        
		if (TargetResourceTransfer.getInstance().isSupportedType(event.dataType)) {
			event.data = resources;
		}
	}

	private ITargetResource[] getResources(ISelection selection) {
		List result = new ArrayList();
		if (selection instanceof IStructuredSelection) {
			for (Iterator iter = ((IStructuredSelection)selection).iterator(); iter.hasNext();) {
				Object element = iter.next();
				ITargetResource resource = Utils.asResource(element);
				if (resource != null) {
					result.add(resource);
				}
			}
		}
		return (ITargetResource[]) result.toArray(new ITargetResource[result.size()]);
	}

	/**
	 * Method declared on DragSourceListener
	 */
	public void dragStart(DragSourceEvent event) {
		event.doit = getResources(viewer.getSelection()).length != 0;
	}
}
